'''spacen
Copyright (c) Jan Varho 2013
See README.txt for licensing
'''

import pygame, math, random, sys, time
from pygame.locals import *

import data

from anim import *
import audio

debug_toggle = 0
debug_color = (255,0,255)

def nazi_target():
    pos = [0,0]
    while pos[0] + pos[1] < 1500:
        pos[0] = random.randrange(1300)
        pos[1] = random.randrange(1300)
    return pos

def spawn_nazi():
    return ([1000, 1000], nazi_target())

def dist2(a, b):
    return (a[0] - b[0]) * (a[0] - b[0]) + (a[1] - b[1]) * (a[1] - b[1])

def vlen2(v):
    return v[0] * v[0] + v[1] * v[1]

def vadd(u, v):
    u[0] += v[0]
    u[1] += v[1]

def vsum(u, v):
    return (u[0] + v[0], u[1] + v[1])

def process_nazis(disp, spos, nazis, nazi, bombs, f, pos):
    for i, n in enumerate(nazis):
        if len(n) == 3 and f > n[2] + 60:
            nazis.remove(n)
            continue
        vel = normalize((n[1][0] - n[0][0], n[1][1] - n[0][1]), 4)
        vadd(n[0], vel)
        if len(n) == 2 and dist2(n[0], n[1]) < 400:
            n[1][0], n[1][1] = nazi_target()
            if dist2(n[0], pos) < 1000000:
                bombs.append((n[0][:], list(vel)))
        npos = vsum(n[0], spos)
        t = f + 42 * i + i * i
        if len(n) == 3:
            r = 0.25 * (f - n[2]) + 0.01 * (f - n[2]) * (f - n[2])
            nazi.draw(disp, npos[0], npos[1], t, r, 1, 60.0 / (60 + f - n[2]))
        else:
            nazi.draw(disp, npos[0], npos[1], t)
        if debug_toggle:
            pygame.draw.circle(disp, debug_color, (int(npos[0]), int(npos[1])), 15)

def process_bullets(disp, spos, bullets, bullet, nazis, f):
    kills = 0
    for b in bullets:
        vadd(b[0], b[1])
        disp.blit(bullet, vsum(b[0], spos))
        if (not -2000 < b[0][0] < 2000) or (not -2000 < b[0][1] < 2000) \
                or vlen2(b[0]) < 1000000:
            bullets.remove(b)
            continue
        
        for n in nazis:
            if len(n) == 3:
                continue
            if coll_obj(n[0], b[0], b[1], 20):
                if debug_toggle:
                    print "Kill"
                bullets.remove(b)
                nazis.remove(n)
                nazis.append((n[0], nazi_target(), f))
                kills += 1
                break
    return kills

def create_explosion(pos, t, ppos, ground = True):
    if not ground:
        return (tuple(pos), t), dist2(ppos, pos) < 900
    if pos[0] == 0:
        ang = 3 * math.pi / 2 if pos[1] > 0 else math.pi / 2
    elif pos[1] == 0:
        ang = 0 if pos[0] > 0 else math.pi
    else:
        ang = math.atan(pos[1] / pos[0])
    pos = (1000 * math.cos(ang), 1000 * math.sin(ang))
    if dist2(ppos, pos) < 900:
        return (pos, t), 1
    else:
        return (pos, t), 0

def process_grenades(disp, spos, grns, grenade, exps, nazis, pos, f):
    kills, hits, grnd = 0, 0, 0
    for g in grns:
        vadd(g[0], g[1])
        disp.blit(grenade, vsum(g[0], spos))
        a = normalize(g[0], -0.1)
        vadd(g[1], a)
        if vlen2(g[0]) < 1000000:
            grns.remove(g)
            e, h = create_explosion(g[0], f, pos, False)
            hits += h
            exps.append(e)
            grnd += 1
            for n in nazis:
                if dist2(n[0], g[0]) < 900:
                    nazis.remove(n)
                    nazis.append((n[0], nazi_target(), f))
                    kills += 1
        
        for n in nazis:
            if len(n) == 3:
                continue
            if coll_obj(n[0], g[0], g[1], 20):
                if debug_toggle:
                    print "Explode"
                grns.remove(g)
                nazis.remove(n)
                kills += 1
                e, h = create_explosion(g[0], f, pos, False)
                hits += h
                exps.append(e)
                for n in nazis:
                    if dist2(n[0], g[0]) < 900:
                        nazis.remove(n)
                        nazis.append((n[0], nazi_target(), f))
                        kills += 1
                break
    return kills, hits, grnd

def draw_exps(disp, spos, exps, expl, t):
    for e in exps:
        if e[1] + 60 <= t:
            exps.remove(e)
            continue
        a = expl[e[1] * 100 % len(expl)]
        a.draw(disp, e[0][0] + spos[0], e[0][1] + spos[1], t - e[1])

def process_bombs(disp, spos, bombs, bomb, pos, exps, t, expl_sound, height):
    hits = 0
    len_exps = len(exps)
    for b in bombs:
        vadd(b[0], b[1])
        vadd(b[1], normalize(b[0], -0.1))
        disp.blit(bomb, vsum(b[0], spos))
        if vlen2(b[0]) < 1000000:
            bombs.remove(b)
            e, h = create_explosion(b[0], t, pos)
            hits += h
            exps.append(e)
            
            continue
        
        if coll_obj(pos, b[0], b[1], 15):
            if debug_toggle:
                print "Hit"
            hits += 1
            bombs.remove(b)
            exps.append((b[0], t))
    if len(exps) > len_exps and (height == 0 or hits):
        audio.play_sound(expl_sound)
    return hits

def draw_stars(disp, stars):
    disp.blit(stars, (0,0))

def draw_me(disp, me, ang, f):
    me.draw(disp, 400, 300, f, ang)

def draw_moon(disp, spos, moon):
    disp.blit(moon, vsum((-1000, -1000), spos))

def coll_obj(pos, bpos, bvel, rad):
    d2 = dist2(pos, bpos)
    vel2 = vlen2(bvel)
    if d2 < rad * rad:
        return 1
    if d2 > vel2 + rad * rad:
        return 0
    
    r2 = math.sqrt(2)
    v1 = bvel[1] if bvel[1] else 0.0000001
    v0v1 = bvel[0] / v1
    l = (bpos[0] - pos[0] + v0v1 * (pos[1] - bpos[1])) / (r2 + v0v1 * 2)
    if l > rad:
        return 0
    
    t = (pos[1] - bpos[1] + l * r2) / v1
    return 0 < t < 1

def hcolor(h):
    c_good = (0, 255, 0)
    c_ok = (255, 255, 0)
    c_poor = (255, 0, 0)
    if h >= 3:
        return c_good
    elif h == 2:
        return c_ok
    return c_poor

def acolor(a):
    c_good = (0, 255, 0)
    c_ok = (255, 255, 0)
    c_poor = (255, 0, 0)
    if a >= 4:
        return c_good
    elif a > 1:
        return c_ok
    return c_poor

def me_fire(bpos, bvel, bullets, ammo, ammo_full, font):
    bullets.append((bpos, bvel))
    if debug_toggle:
        print bpos, bvel
    ammo -= 1
    return ammo, font.render("Ammo: %d/%d" % (ammo, ammo_full), True, acolor(ammo))

def me_throw(bpos, bvel, bullets, grenades, font):
    bullets.append((bpos, bvel, 2))
    if debug_toggle:
        print bpos
    grenades -= 1
    if not grenades:
        return 0, None
    return grenades, font.render("Special: %d grenade%s" %
                        (grenades, 's' if grenades > 1 else ''), True, gui_fg)

def spawn_powerup(t):
    ang = random.uniform(math.pi, 1.5 * math.pi)
    uvel = (math.cos(ang), math.sin(ang))
    height = random.randrange(500, 1000)
    pup_type = random.randrange(1,3)
    tmp = random.randrange(1,13)
    if tmp <= 4:
        pup_type = 1
    elif tmp <= 10:
        pup_type = 2
    elif tmp == 11:
        pup_type = 3
    elif tmp == 12:
        pup_type = 4
    return [ang, height, 0.1, uvel, pup_type, t]

def process_powerups(disp, spos, pups, pos, pup_img, t):
    caught = 0
    for p in pups:
        ppos = ((1000 + p[1]) * math.cos(p[0]), (1000 + p[1]) * math.sin(p[0]))
        pvel = (p[3][0] * p[2], p[3][1] * p[2])
        if dist2(ppos, pos) < 400:
            pups.remove(p)
            caught = p[4]
            continue
        p[1] -= p[2]
        if p[1] < 0:
            p[1] = 0
            if p[2] > 3:
                p[2] = -p[2] / 3
            else:
                p[2] = 0
        p[2] += 0.1
        alpha = None
        if t > p[5] + 600:
            if t > p[5] + 900:
                pups.remove(p)
                continue
            if (t - p[5]) % 60 < 30:
                continue
        pup_img[p[4] - 1].draw(disp, int(ppos[0]) + spos[0],
                                     int(ppos[1]) + spos[1], 0, p[0])
    return caught

def normalize(vec, mul = None):
    d = math.sqrt(vec[0]*vec[0] + vec[1]*vec[1])
    if mul is not None:
        mul /= d
    else:
        mul = 1.0 / d
    return (mul * vec[0], mul * vec[1])

def main():
    global debug_toggle
    
    audio.preinit()
    pygame.init()
    disp = pygame.display.set_mode((800, 600))
    pygame.display.set_caption("spacen")
    bg = (0,) * 3
    global fg
    fg = (128,) * 3
    gui_bg = fg
    global gui_fg
    gui_fg = (255,) * 3
    font = pygame.font.Font('freesansbold.ttf', 16)
    font_large = pygame.font.Font('freesansbold.ttf', 32)
    
    try:
        audio.play_music(data.load('perpetual-tension.ogg'))
        audio.set_music_volume(0.7)
    except IOError:
        print "No music available!"
    
    pistol_shot = audio.load_sound(data.load('pistol-shot.ogg'))
    pistol_reload = audio.load_sound(data.load('pistol-reload.ogg'))
    grunt = [audio.load_sound(data.load('pain%d.ogg' % i)) for i in range(1, 7)]
    powerup = audio.load_sound(data.load('pickup.ogg'), 0.4)
    jump = (audio.load_sound(data.load('jump.ogg'), 0.5),
            audio.load_sound(data.load('land.ogg'), 0.5))
    grenade_throw = audio.load_sound(data.load('pickup.ogg'), 0.6)
    chainsaw_running = audio.load_sound(data.load('chainsaw-running.ogg'))
    chainsaw_cutting = audio.load_sound(data.load('chainsaw-cutting.ogg'))
    
    stars = pygame.image.load(data.load('stars.png')).convert()

    moon = pygame.image.load(data.load('moon.png')).convert_alpha()

    me_base = pygame.image.load(data.load('hero.png'))
    me_r = Anim(me_base.subsurface(Rect((0, 180), (240, 90))), 6, 1, 6)
    me_l = me_r.flipped()
    me_ur = Anim(me_base.subsurface(Rect((0, 360), (240, 90))), 6, 1, 6)
    me_ul = me_ur.flipped()
    me_hr = Anim(me_base.subsurface(Rect((40, 270), (40, 90))))
    me_hl = me_hr.flipped()
    me_uhr = Anim(me_base.subsurface(Rect((240, 360), (40, 90))))
    me_uhl = me_uhr.flipped()
    me = me_r
    
    chainsaw = 0
    mec_base = pygame.image.load(data.load('hero-chainsaw.png'))
    mec_r = Anim(mec_base.subsurface(Rect((0, 180), (240, 90))), 6, 1, 6)
    mec_l = mec_r.flipped()
    mec_hr = Anim(mec_base.subsurface(Rect((40, 270), (40, 90))))
    mec_hl = mec_hr.flipped()
    
    ang = 3*math.pi / 2
    height = 0
    speed = 0
    
    bullet = pygame.Surface((2,2))
    bullet.fill((128,128,128))
    bullets = []
    
    nazi = Anim(pygame.image.load(data.load('nazi.png')), 5, 1, 5)
    nazis = [spawn_nazi()]

    bomb = pygame.Surface((2, 2))
    bomb.fill((255,0,0))
    bombs = []
    
    expl = Anim(pygame.image.load(data.load('expl.png')), 4, 4, 16)
    expl = [expl.rotated(i) for i in xrange(0, 360, 45)]
    expl_sound = audio.load_sound(data.load('explosion-on-moon.ogg'))
    exps = []
    
    pup = pygame.image.load(data.load('powerup.png'))
    pup_health = Anim(pup.subsurface(Rect((0,0), (32,64))))
    pup_grenade = Anim(pup.subsurface(Rect((32,0), (32,64))))
    pup_exclip = Anim(pup.subsurface(Rect((96,0), (32,64))))
    pup_chainsaw = Anim(pup.subsurface(Rect((128,0), (32,64))))
    pup_img = [pup_health, pup_grenade, pup_exclip, pup_chainsaw]
    pup_ammo = pup.subsurface(Rect((64,0), (32,64)))
    pup_ammo_intro = Anim(pup_ammo)
    pup_ammo = pygame.transform.rotate(pup_ammo, 0.3 * 180)
    pup_ammo_center = pup_ammo.get_rect().center
    pup_ammo_pos = (int(1000 * math.cos(1.2 * math.pi) - pup_ammo_center[0]),
                    int(1000 * math.sin(1.2 * math.pi) - pup_ammo_center[1]))
    pups = []

    reloading = 0
    text_reload = font_large.render("RELOADING...", True, fg)
    
    health = 3
    d_health = font.render("Health: %d" % health, True, hcolor(health))
    
    ammo = 6
    ammo_full = 6
    grenades = 0
    d_ammo = font.render("Ammo: %d/%d" % (ammo, ammo_full), True, acolor(ammo))
    d_special = None
    grenade = pygame.Surface((3,3))
    grenade.fill((60,70,40))
    grns = []

    kills = 0
    d_kills = font.render("Kills: %d" % kills, True, gui_fg)

    stat_hits = 0
    stat_pups = 0
    stat_shot = 0
    stat_chain = 0

    d_gui = pygame.Surface((800,100))
    d_gui.fill(gui_bg)
    d_gui.set_alpha(128)

    d_round = font.render("Round: %d" % 0, True, gui_fg)

    fps = pygame.time.Clock()
    frames = 0
    start = time.time()
    frames_per_round = 2400

    mouse_x, mouse_y = 0.0, 0.0

    #intro loop
    intro = pygame.Surface((800, 400))
    intro.fill((128,128,128))
    intro.set_alpha(192)
    introtext = font.render('You are International Antiterror Special Ops '
        'soldier Lt. Charles, sent to investigate reported Space Nazi', True, bg)
        
    introtext2 = font.render('activity near the moon.  You find a station of '
        'theirs on the dark side of the moon, when they manage to', True, bg)
    introtext3 = font.render('damage your ship and you are forced to land. '
        'Your new mission is to get as many of those bastards as', True, bg)
    introtext4 = font.render('you can. Irregular shipments of equipment will '
        'be sent to the light side of the moon, where your ammo', True, bg)
    introtext5 = font.render('cache already is.', True, bg)
    introtext_eq = font.render('            Ammo Cache  Heath Pack      '
        'Grenade             You           Space Nazi  Extended Mag'
        ' Chainsaw', True, bg)
    introtext_keys = font.render('Left Mouse = Fire,   Right Mouse = Special '
        'Weapon,   Space = Jump.  Press any key to start...', True, bg)
    escape = False
    while not escape:
        ang = (frames - 600) * 2 * math.pi / frames_per_round 
        if ang > math.pi:
            ang = ang - 2 * math.pi
        pos = ((1015 + height) * math.cos(ang), (1015 + height) * math.sin(ang))
        spos = (400 - (1000 + height) * math.cos(ang), 300 - (1000 + height) * math.sin(ang))
        
        for event in pygame.event.get():
            if event.type == QUIT or (event.type == KEYDOWN
                                      and event.key == K_q):
                pygame.quit()
                sys.exit()
            if event.type == KEYDOWN:
                escape = True
        
        draw_stars(disp, stars)
        draw_moon(disp, spos, moon)
        
        disp.blit(d_health, (20, 20))
        disp.blit(d_ammo, (20, 40))
        disp.blit(d_kills, (20, 60))
        
        disp.blit(intro, (0,100))
        disp.blit(introtext, (5,105))
        disp.blit(introtext2, (5,125))
        disp.blit(introtext3, (5,145))
        disp.blit(introtext4, (5,165))
        disp.blit(introtext5, (5,185))
        
        pup_ammo_intro.draw(disp, 100, 300, 0)
        pup_health.draw(disp, 200, 300, 0)
        pup_grenade.draw(disp, 300, 300, 0)
        draw_me(disp, me, ang, frames)
        nazi.draw(disp, 500, 280, 0)
        pup_exclip.draw(disp, 600, 300, 0)
        pup_chainsaw.draw(disp, 700, 300, 0)
        
        disp.blit(introtext_eq, (0, 350))
        disp.blit(introtext_keys, (5, 450))
        
        pygame.display.update()
        fps.tick(60)
        
    # Main loop
    start = time.time()
    while health > 0 or debug_toggle:
        frames += 1
        round_id, frame_id = divmod(frames, frames_per_round)
        
        ang = (frame_id - 600) * 2 * math.pi / frames_per_round 
        if ang > math.pi:
            ang = ang - 2 * math.pi
        if height or speed:
            height += speed
            speed -= 0.1
            if height < 0:
                audio.play_sound(jump[1])
                height = 0
                speed = 0
        pos = ((1015 + height) * math.cos(ang), (1015 + height) * math.sin(ang))
        spos = (400 - (1000 + height) * math.cos(ang), 300 - (1000 + height) * math.sin(ang))
        
        if frame_id == int(0.85 * frames_per_round) and ammo < ammo_full \
                                                    and height < 15:
            reloading = 1
            audio.play_sound(pistol_reload, 6 - ammo)
            d_ammo = font.render("Ammo: reloading...", True, gui_fg)
            ammo = 0
            for i in range(random.randrange(1, 2 + frames // 1000)):
                nazis.append(spawn_nazi())
        elif reloading and (frame_id > int(0.95 * frames_per_round) \
                            or frame_id < int(0.85 * frames_per_round)):
            reloading = 0
            ammo = ammo_full
            d_ammo = font.render("Ammo: %d/%d" % (ammo, ammo_full), True, acolor(ammo))
        elif frame_id == int(0.7 * frames_per_round):
            if random.randrange(0, 1 + round_id) == 0:
                pups.append(spawn_powerup(frames))
                if debug_toggle:
                    print "Powerup spawned: %d!" % pups[-1][4]
            elif debug_toggle:
                print "No powerup spawned!"
        elif frame_id == 1:
            d_round = font.render("Round: %d" % (round_id + 1), True, gui_fg)
        
        for event in pygame.event.get():
            if event.type == KEYDOWN:
                if debug_toggle:
                    print event
                if event.key == K_SPACE and height == 0:
                    speed = 5
                    audio.play_sound(jump[0])
                if True: # "Commenting out" debug options
                    pass
                elif event.key == K_z:
                    nazis.append(spawn_nazi())
                elif event.key == K_x:
                    debug_toggle = not debug_toggle
                elif event.key == K_c:
                    frames += 500
                elif event.key == K_v:
                    pups.append(spawn_powerup(frames))
                    if debug_toggle:
                        print "Powerup spawned: %d!" % pups[-1][4]
                elif event.key == K_a:
                    ammo = 1000
                elif event.key == K_g:
                    grenades += 3
                elif event.key == K_d:
                    health = 0
                elif event.key == K_s:
                    grenades = 0
                    chainsaw = 1
            if event.type == MOUSEBUTTONDOWN:
                if debug_toggle:
                    print event
                bpos = [pos[0], pos[1]]
                bvel = (event.pos[0] - (spos[0] + pos[0]),
                        event.pos[1] - (spos[1] + pos[1]))
                if event.button == 1:
                    if ammo:
                        bvel = normalize(bvel, 15.0)
                        stat_shot += 1
                        ammo, d_ammo = me_fire(bpos, bvel, bullets, ammo, ammo_full, font)
                        audio.play_sound(pistol_shot)
                elif event.button == 3:
                    if grenades:
                        bvel = list(normalize(bvel, 7.0))
                        grenades, d_special = me_throw(bpos, bvel, grns, grenades, font)
                        audio.play_sound(grenade_throw)
                    if chainsaw:
                        audio.play_sound(chainsaw_running, -1, 300)
                elif event.button == 2:
                    mpos = (mouse_x - spos[0], mouse_y - spos[1])
                    a = math.atan2(mpos[1], mpos[0])
                    h = mpos[0] / math.cos(a)
                    b = math.atan2(mouse_y - 300, mouse_x - 400) - a
                    if b > math.pi:
                        b -= 2 * math.pi
                    elif b < -math.pi:
                        b += 2 * math.pi
                    print pos, mpos, a, ang, b
            if event.type == MOUSEBUTTONUP and event.button == 3 and chainsaw:
                audio.stop_sound(chainsaw_running, 200)
            if event.type == MOUSEMOTION:
                mouse_x, mouse_y = event.pos
            if event.type == QUIT or \
                    (event.type == KEYDOWN and event.key == K_q) or \
                    (event.type == KEYDOWN and event.key == K_ESCAPE) :
                pygame.quit()
                sys.exit()
        
        if height > 0:
            mpos = (mouse_x - spos[0], mouse_y - spos[1])
            a = math.atan2(mpos[1], mpos[0])
            b = math.atan2(mouse_y - 300, mouse_x - 400) - a
            a -= ang
            
            if a > math.pi:
                a -= 2 * math.pi
            elif a < -math.pi:
                a += 2 * math.pi
            if b > math.pi:
                b -= 2 * math.pi
            elif b < -math.pi:
                b += 2 * math.pi
            
            if a > 0 and b < 1:
                me = me_uhr
            elif a > 0:
                me = me_hr
            elif b > -1:
                me = me_uhl
            else:
                me = me_hl
        else:
            mpos = (mouse_x - spos[0], mouse_y - spos[1])
            a = math.atan2(mpos[1], mpos[0])
            b = math.atan2(mouse_y - 300, mouse_x - 400) - a
            a -= ang
            
            if a > math.pi:
                a -= 2 * math.pi
            elif a < -math.pi:
                a += 2 * math.pi
            if b > math.pi:
                b -= 2 * math.pi
            elif b < -math.pi:
                b += 2 * math.pi
            
            if a > 0 and b < 1:
                me = me_ur
            elif a > 0:
                me = me_r
            elif b > -1:
                me = me_ul
            else:
                me = me_l
        
        if chainsaw and pygame.mouse.get_pressed()[2]:
            if me == me_l or me == me_ul:
                me = mec_l
            elif me == me_r or me == me_ur:
                me = mec_r
            elif me == me_hl or me == me_uhl:
                me = mec_hl
            else:
                me = mec_hr
            for n in nazis:
                if dist2(n[0], pos) < 250:
                    nazis.remove(n)
                    nazis.append((n[0], nazi_target(), frames))
                    kills += 1
                    stat_chain += 1
                    audio.play_sound(chainsaw_cutting)
                    
        
        t = time.time()
        
        draw_stars(disp, stars)
        
        disp.blit(pup_ammo, vsum(pup_ammo_pos, spos))
        c = process_powerups(disp, spos, pups, pos, pup_img, frames)
        if c:
            stat_pups += 1
            if debug_toggle:
                print "Powerup picked up: %d!" % c
            if c == 1:
                health += 1
                d_health = font.render("Health: %d" % health, True, hcolor(health))
            elif c == 2:
                grenades += 1
                chainsaw = 0
                d_special = font.render("Special: %d grenade%s" %
                            (grenades, 's' if grenades > 1 else ''), True, gui_fg)
            elif c == 3:
                ammo_full = 9
                d_ammo = font.render("Ammo: %d/%d" % (ammo, ammo_full), True, acolor(ammo))
            elif c == 4:
                grenades = 0
                chainsaw = 1
                d_special = font.render("Special: chainsaw", True, gui_fg)
            audio.play_sound(powerup)
        
        draw_me(disp, me, ang, frames)
        if debug_toggle:
            pygame.draw.circle(disp, debug_color,
                            (int(pos[0] + spos[0]), int(pos[1] + spos[1])), 15)

        process_nazis(disp, spos, nazis, nazi, bombs, frames, pos)
        
        k = process_bullets(disp, spos, bullets, bullet, nazis, frames)
        if k:
            kills += k
            d_kills = font.render("Kills: %d" % kills, True, gui_fg)
        
        k, h, g = process_grenades(disp, spos, grns, grenade, exps, nazis, pos, frames)
        if k:
            kills += k
            d_kills = font.render("Kills: %d" % kills, True, gui_fg)
        if h:
            health -= h
            stat_hits += 1
            d_health = font.render("Health: %d" % health, True, hcolor(health))
            audio.play_sound(random.choice(grunt))
            audio.play_sound(expl_sound)
        if g and not h and height == 0:
            audio.play_sound(expl_sound)
        
        h = process_bombs(disp, spos, bombs, bomb, pos, exps, frames, expl_sound, height)
        if h:
            health -= h
            stat_hits += 1
            d_health = font.render("Health: %d" % health, True, hcolor(health))
            audio.play_sound(random.choice(grunt))
        
        draw_moon(disp, spos, moon)
        
        draw_exps(disp, spos, exps, expl, frames)
        
        disp.blit(d_gui, (0,0))
        if debug_toggle:
            t2 = font.render("Frame %d, %d/%d, %d FPS, %f, %d, %d, %d" % (frames, frame_id, int(frames_per_round), int(frames / (t - start)), height, len(bullets), len(nazis), len(exps)), True, fg)
            disp.blit(t2, (0,0))
        disp.blit(d_health, (20, 20))
        disp.blit(d_ammo, (20, 40))
        disp.blit(d_kills, (20, 60))
        disp.blit(d_round, (320, 20))
        if d_special:
            disp.blit(d_special, (320, 40))
        if reloading:
            disp.blit(text_reload, (400 - text_reload.get_rect().center[0],
                                    300 - text_reload.get_rect().center[1]))
        
        pygame.display.update()
        fps.tick(60)
    
    # Outro
    disp.blit(intro, (0,100))
    gameover = font_large.render('Game Over', True, (255,0,0))
    disp.blit(gameover, (400 - gameover.get_rect().center[0], 105))
    credits = font.render('Thanks to the following artists for sharing their '
        'work on OpenGameArt.org or Freesound.org: AngryMeteor.com, AWV, '
        'Cusco, LFA, lonemonk, Michel Baradari, Nicolas Mina, Tokka and Zander '
        'Noriega', True, fg, bg)
    bgslice = pygame.Surface((800, credits.get_rect().height))
    bgslice.fill(bg)
    pygame.display.update()
    stat_rounds = int(frames / frames_per_round)
    frames = 0
    delay = 120
    
    stat_lines = [
        font.render('You circled the moon %d times...' % stat_rounds, True, bg),
        font.render('You killed %d space nazis...' % kills, True, bg),
        font.render('You got hurt %d times...' % stat_hits, True, bg),
        font.render('You used %d bullets...' % stat_shot, True, bg),
        font.render('You picked up %d items...' % stat_pups, True, bg),
        font.render('You died.', True, bg)
    ]
    
    stat_prog = 1
    
    if stat_chain:
        stat_lines.insert(2, font.render(
            'You used the chainsaw %d times...' % stat_chain, True, bg))
    
    while True:
        for event in pygame.event.get():
            if event.type == QUIT or (event.type == KEYDOWN and frames >= 60):
                pygame.quit()
                sys.exit()
        
        if len(stat_lines) and frames == stat_prog * delay:
            disp.blit(stat_lines.pop(0), (105, 180 + stat_prog * 20))
            stat_prog += 1
        elif frames == stat_prog * delay:
            if kills == 0 or stat_rounds < 3:
                verdict = font_large.render('You suck!', True, (255,0,0))
            elif kills == 42 or stat_rounds == 42:
                verdict = font_large.render('Six by Nine.', True, (0,0,255))
            elif kills < 10 or stat_rounds < 10:
                verdict = font_large.render('You did fine, for a beginner!',
                                            True, (255,255,0))
            elif stat_chain >= kills // 2:
                verdict = font_large.render('Chainsaw Massacre',
                                            True, (175,17,28))
            else:
                verdict = font_large.render('You rock!', True, (0,255,0))
            disp.blit(verdict, (400 - verdict.get_rect().center[0],
                                495 - verdict.get_rect().height))
        
        #credits
        x = 800 - frames
        while x < -credits.get_rect().width:
            x += 800 + credits.get_rect().width
        disp.blit(bgslice, (0, 600 - credits.get_rect().height))
        disp.blit(credits, (x, 600 - credits.get_rect().height))
        pygame.display.update()
        fps.tick(60)
        frames += 1

    
